/*@@
  @file    LapseSource.F
  @date    February 2001
  @author  Miguel Alcubierre
  @desc
     Source term for lapse evolution equation.
  @enddesc
  @version $Header: /numrelcvs/AEIThorns/BSSN_MoL/src/LapseSource.F,v 1.8 2003/09/29 15:25:41 tradke Exp $
@@*/

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"

      subroutine ADM_BSSN_LapseSource(CCTK_ARGUMENTS)

      implicit none 

      DECLARE_CCTK_ARGUMENTS
      DECLARE_CCTK_PARAMETERS

      integer :: i,j,k,nx,ny,nz
      integer :: CCTK_Equals

      CCTK_REAL :: zero,one,two
      CCTK_REAL :: scut,aux

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing_declare.h"

c      CCTK_REAL, DIMENSION(cctk_lsh(1),cctk_lsh(2),cctk_lsh(3)) :: adm_bs_gauge
      CCTK_REAL, allocatable, DIMENSION(:,:,:) :: adm_bs_gauge

c     Numbers.

      zero = 0.0D0
      one  = 1.0D0
      two  = 2.0D0

c     Allocate local array.

      allocate(adm_bs_gauge(cctk_lsh(1),cctk_lsh(2),cctk_lsh(3)))

      if (LapsePsiPower == 0) then
        scut = one
      else
         scut = LapsePsiPower_cut**(one/LapsePsiPower)
      end if

c     Grid parameters.

      nx = cctk_lsh(1)
      ny = cctk_lsh(2)
      nz = cctk_lsh(3)

#include "CactusEinstein/ADMMacros/src/macro/ADM_Spacing.h"

c     Construct array adm_bs_gauge.

      call ADM_BSSN_Lapse(CCTK_ARGUMENTS,adm_bs_gauge)

c     Calculate lapse source.

      if (CCTK_Equals(lapsesource,"standard") == 1) then

c        Here the source for the lapse is just proportional to
c        the source for phi.

         adm_bs_sdtalp = zero

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  aux = 6.0D0*adm_bs_gauge(i,j,k)

                  aux = aux*exp(LapsePhiPower*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux = aux*(one+scut*(psi(i,j,k)-one))**LapsePsiPower
                  end if

                  adm_bs_salp(i,j,k) = aux*adm_bs_sphi(i,j,k)

               end do
            end do
         end do

      else

c        Here the first time derivative of the lapse is proportional
c        to (trK - trK(t=0)).

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  aux = - alp(i,j,k)*adm_bs_gauge(i,j,k)

                  aux = aux*exp(LapsePhiPower*adm_bs_phi(i,j,k))

                  if (conformal_state == 3) then
                     aux = aux*(one+scut*(psi(i,j,k)-one))**LapsePsiPower
                  end if

                  adm_bs_salp(i,j,k) = aux*ADM_BS_dtalp(i,j,k)

                  adm_bs_sdtalp(i,j,k) = adm_bs_sK(i,j,k)
     &               - AlphaDriver*adm_bs_dtalp(i,j,k)

               end do
            end do
         end do

      end if

c     Add dissipation.

      if (AlphaDissip.ne.zero) then

         do k=2,nz-1
            do j=2,ny-1
               do i=2,nx-1

                  adm_bs_salp(i,j,k) = adm_bs_salp(i,j,k) + dt*AlphaDissip*
     &              ((alp(i+1,j,k) + alp(i-1,j,k) - two*alp(i,j,k))*idxx
     &              +(alp(i,j+1,k) + alp(i,j-1,k) - two*alp(i,j,k))*idyy
     &              +(alp(i,j,k+1) + alp(i,j,k-1) - two*alp(i,j,k))*idzz)

               end do
            end do
         end do

      end if

c     free local array

      deallocate(adm_bs_gauge)

c     End.

      end subroutine ADM_BSSN_LapseSource


