/*@@
  @header   BSRICCI_guts.h
  @date     Nov 98
  @author   Gabrielle Allen
  @desc
  Macro to the components of the Ricci tensor using the BS 
  variables and Eq. 22 of the BS paper. 

  @enddesc
@@*/

#ifndef INSOURCES
/* This macro is being called from outside of Sources.F */
#ifdef OPT
  OPT is defined while not in Sources.F
#endif
#undef OPT
#endif

#ifndef BSRICCI_GUTS
#define BSRICCI_GUTS

#include "BSCHR1_guts.h"
#include "BSCHR2_guts.h"
#include "WAVEBSG_guts.h"
#include "BSGAMMA_guts.h"

#include "BSSN_Derivative.h"
      if (useGamma.eq.1) then

c     BSRICCI_GX = ADM_BS_Gx(i,j,k)
c     BSRICCI_GY = ADM_BS_Gy(i,j,k)
c     BSRICCI_GZ = ADM_BS_Gz(i,j,k)

      BSRICCI_GX = BSGAMMA_GAMMAX
      BSRICCI_GY = BSGAMMA_GAMMAY
      BSRICCI_GZ = BSGAMMA_GAMMAZ

      if (local_spatial_order.eq.2) then
        BSRICCI_DXGX = BSSN_DX_2(ADM_BS_Gx,i,j,k)
        BSRICCI_DYGX = BSSN_DY_2(ADM_BS_Gx,i,j,k)
        BSRICCI_DZGX = BSSN_DZ_2(ADM_BS_Gx,i,j,k)

        BSRICCI_DXGY = BSSN_DX_2(ADM_BS_Gy,i,j,k)
        BSRICCI_DYGY = BSSN_DY_2(ADM_BS_Gy,i,j,k)
        BSRICCI_DZGY = BSSN_DZ_2(ADM_BS_Gy,i,j,k)

        BSRICCI_DXGZ = BSSN_DX_2(ADM_BS_Gz,i,j,k)
        BSRICCI_DYGZ = BSSN_DY_2(ADM_BS_Gz,i,j,k)
        BSRICCI_DZGZ = BSSN_DZ_2(ADM_BS_Gz,i,j,k)
      else
        BSRICCI_DXGX = BSSN_DX_4(ADM_BS_Gx,i,j,k)
        BSRICCI_DYGX = BSSN_DY_4(ADM_BS_Gx,i,j,k)
        BSRICCI_DZGX = BSSN_DZ_4(ADM_BS_Gx,i,j,k)

        BSRICCI_DXGY = BSSN_DX_4(ADM_BS_Gy,i,j,k)
        BSRICCI_DYGY = BSSN_DY_4(ADM_BS_Gy,i,j,k)
        BSRICCI_DZGY = BSSN_DZ_4(ADM_BS_Gy,i,j,k)

        BSRICCI_DXGZ = BSSN_DX_4(ADM_BS_Gz,i,j,k)
        BSRICCI_DYGZ = BSSN_DY_4(ADM_BS_Gz,i,j,k)
        BSRICCI_DZGZ = BSSN_DZ_4(ADM_BS_Gz,i,j,k)
      end if

      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGXX

#ifdef OPT
      BSRICCI_TERM2 =  lg(XX,i,j,kc)*BSRICCI_DXGX
     &                +lg(XY,i,j,kc)*BSRICCI_DXGY 
     &                +lg(XZ,i,j,kc)*BSRICCI_DXGZ 
#else
      BSRICCI_TERM2 =  ADM_BS_gxx(i,j,k)*BSRICCI_DXGX
     &                +ADM_BS_gxy(i,j,k)*BSRICCI_DXGY 
     &                +ADM_BS_gxz(i,j,k)*BSRICCI_DXGZ 
#endif
      BSRICCI_TERM3 =  BSRICCI_GX*BSCHR1_XXX
     &                +BSRICCI_GY*BSCHR1_XXY
     &                +BSRICCI_GZ*BSCHR1_XXZ
      BSRICCI_RXX = BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3 + 
     & ( 3*BSCHR2_XXX*BSCHR1_XXX + 
     & 2*(BSCHR2_YXX*BSCHR1_XXY + BSCHR2_ZXX*BSCHR1_XXZ) + 
     & BSCHR2_YXX*BSCHR1_YXX + BSCHR2_ZXX*BSCHR1_ZXX)*BSUPPERMET_UXX + 
     & ( 3*(BSCHR2_XXY*BSCHR1_XXX + BSCHR2_XXX*BSCHR1_XXY) + 
     & 2*(BSCHR2_YXY*BSCHR1_XXY + BSCHR2_ZXY*BSCHR1_XXZ + 
     & BSCHR2_YXX*BSCHR1_XYY + BSCHR2_ZXX*BSCHR1_XYZ) + 
     & BSCHR2_YXY*BSCHR1_YXX + BSCHR2_YXX*BSCHR1_YXY + 
     & BSCHR2_ZXY*BSCHR1_ZXX + BSCHR2_ZXX*BSCHR1_ZXY)*BSUPPERMET_UXY + 
     & (3*(BSCHR2_XXZ*BSCHR1_XXX + BSCHR2_XXX*BSCHR1_XXZ) + 
     & 2*(BSCHR2_YXZ*BSCHR1_XXY + BSCHR2_ZXZ*BSCHR1_XXZ + 
     & BSCHR2_YXX*BSCHR1_XYZ + BSCHR2_ZXX*BSCHR1_XZZ) + 
     & BSCHR2_YXZ*BSCHR1_YXX + BSCHR2_YXX*BSCHR1_YXZ + 
     & BSCHR2_ZXZ*BSCHR1_ZXX + BSCHR2_ZXX*BSCHR1_ZXZ)*BSUPPERMET_UXZ + 
     & (3*BSCHR2_XXY*BSCHR1_XXY + 2*(BSCHR2_YXY*BSCHR1_XYY + 
     & BSCHR2_ZXY*BSCHR1_XYZ) + BSCHR2_YXY*BSCHR1_YXY + 
     & BSCHR2_ZXY*BSCHR1_ZXY)*BSUPPERMET_UYY + 
     & (3*(BSCHR2_XXZ*BSCHR1_XXY + BSCHR2_XXY*BSCHR1_XXZ) + 
     & 2*(BSCHR2_YXZ*BSCHR1_XYY + (BSCHR2_YXY + BSCHR2_ZXZ)*BSCHR1_XYZ + 
     & BSCHR2_ZXY*BSCHR1_XZZ) + BSCHR2_YXZ*BSCHR1_YXY + 
     & BSCHR2_YXY*BSCHR1_YXZ + BSCHR2_ZXZ*BSCHR1_ZXY + 
     & BSCHR2_ZXY*BSCHR1_ZXZ)*BSUPPERMET_UYZ + (3*BSCHR2_XXZ*BSCHR1_XXZ + 
     & 2*(BSCHR2_YXZ*BSCHR1_XYZ + BSCHR2_ZXZ*BSCHR1_XZZ) + 
     & BSCHR2_YXZ*BSCHR1_YXZ + BSCHR2_ZXZ*BSCHR1_ZXZ)*BSUPPERMET_UZZ 

      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGXY
#ifdef OPT
      BSRICCI_TERM2 =  0.5D0*(lg(XX,i,j,kc)*BSRICCI_DYGX
     &                       +lg(XY,i,j,kc)*BSRICCI_DYGY 
     &                       +lg(XZ,i,j,kc)*BSRICCI_DYGZ 
     &                       +lg(XY,i,j,kc)*BSRICCI_DXGX
     &                       +lg(YY,i,j,kc)*BSRICCI_DXGY 
     &                       +lg(YZ,i,j,kc)*BSRICCI_DXGZ)
#else
      BSRICCI_TERM2 =  0.5D0*(ADM_BS_gxx(i,j,k)*BSRICCI_DYGX
     &                       +ADM_BS_gxy(i,j,k)*BSRICCI_DYGY 
     &                       +ADM_BS_gxz(i,j,k)*BSRICCI_DYGZ 
     &                       +ADM_BS_gxy(i,j,k)*BSRICCI_DXGX
     &                       +ADM_BS_gyy(i,j,k)*BSRICCI_DXGY 
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DXGZ)
#endif
      BSRICCI_TERM3 =  0.5D0*(BSRICCI_GX*BSCHR1_XYX
     &                       +BSRICCI_GY*BSCHR1_XYY
     &                       +BSRICCI_GZ*BSCHR1_XYZ
     &                       +BSRICCI_GX*BSCHR1_YXX
     &                       +BSRICCI_GY*BSCHR1_YXY
     &                       +BSRICCI_GZ*BSCHR1_YXZ)
      BSRICCI_RXY = BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3 +
     & (BSCHR2_XXY*BSCHR1_XXX + (BSCHR2_XXX + BSCHR2_YXY)*BSCHR1_XXY + 
     & BSCHR2_ZXY*BSCHR1_XXZ + BSCHR2_XXX*BSCHR1_YXX + 2*BSCHR2_YXX*BSCHR1_YXY
     & + BSCHR2_ZXX*(BSCHR1_YXZ + BSCHR1_ZXY))*BSUPPERMET_UXX + 
     & (BSCHR2_XYY*BSCHR1_XXX + (2*BSCHR2_XXY + BSCHR2_YYY)*BSCHR1_XXY + 
     & BSCHR2_ZYY*BSCHR1_XXZ + (BSCHR2_XXX + BSCHR2_YXY)*BSCHR1_XYY + 
     & BSCHR2_XXY*BSCHR1_YXX + (BSCHR2_XXX + 2*BSCHR2_YXY)*BSCHR1_YXY + 
     & 2*BSCHR2_YXX*BSCHR1_YYY + BSCHR2_ZXY*
     &  (BSCHR1_XYZ + BSCHR1_YXZ + BSCHR1_ZXY) + 
     & BSCHR2_ZXX*(BSCHR1_YYZ + BSCHR1_ZYY))*BSUPPERMET_UXY + 
     & (BSCHR2_XYZ*BSCHR1_XXX + (BSCHR2_XXZ + BSCHR2_YYZ)*BSCHR1_XXY + 
     & (BSCHR2_XXY + BSCHR2_ZYZ)*BSCHR1_XXZ + (BSCHR2_XXX + BSCHR2_YXY)*BSCHR1_XYZ + 
     & BSCHR2_ZXY*BSCHR1_XZZ + BSCHR2_XXZ*BSCHR1_YXX + 
     & (BSCHR2_XXX + BSCHR2_ZXZ)*BSCHR1_YXZ + 
     & 2*(BSCHR2_YXZ*BSCHR1_YXY + BSCHR2_YXX*BSCHR1_YYZ) + BSCHR2_ZXZ*BSCHR1_ZXY + 
     & BSCHR2_ZXX*(BSCHR1_YZZ + BSCHR1_ZYZ))*BSUPPERMET_UXZ + 
     & (BSCHR2_XYY*BSCHR1_XXY + 
     & (BSCHR2_XXY + BSCHR2_YYY)*BSCHR1_XYY + BSCHR2_ZYY*BSCHR1_XYZ + 
     & BSCHR2_XXY*BSCHR1_YXY + 2*BSCHR2_YXY*BSCHR1_YYY + 
     & BSCHR2_ZXY*(BSCHR1_YYZ + BSCHR1_ZYY))*BSUPPERMET_UYY + 
     & (BSCHR2_XYZ*BSCHR1_XXY + BSCHR2_XYY*BSCHR1_XXZ + 
     & (BSCHR2_XXZ + BSCHR2_YYZ)*BSCHR1_XYY + 
     & (BSCHR2_XXY + BSCHR2_YYY + BSCHR2_ZYZ)*BSCHR1_XYZ + BSCHR2_ZYY*BSCHR1_XZZ + 
     & BSCHR2_XXZ*BSCHR1_YXY + BSCHR2_XXY*BSCHR1_YXZ + 
     & 2*(BSCHR2_YXZ*BSCHR1_YYY + BSCHR2_YXY*BSCHR1_YYZ) + 
     & BSCHR2_ZXZ*(BSCHR1_YYZ + BSCHR1_ZYY) + BSCHR2_ZXY*(BSCHR1_YZZ + BSCHR1_ZYZ))
     & *BSUPPERMET_UYZ + (BSCHR2_XYZ*BSCHR1_XXZ + 
     & (BSCHR2_XXZ + BSCHR2_YYZ)*BSCHR1_XYZ + BSCHR2_ZYZ*BSCHR1_XZZ + 
     & BSCHR2_XXZ*BSCHR1_YXZ + 2*BSCHR2_YXZ*BSCHR1_YYZ + 
     & BSCHR2_ZXZ*(BSCHR1_YZZ + BSCHR1_ZYZ))*BSUPPERMET_UZZ


      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGXZ
#ifdef OPT
      BSRICCI_TERM2 =  0.5D0*(lg(XX,i,j,kc)*BSRICCI_DZGX
     &                       +lg(XY,i,j,kc)*BSRICCI_DZGY 
     &                       +lg(XZ,i,j,kc)*BSRICCI_DZGZ 
     &                       +lg(XZ,i,j,kc)*BSRICCI_DXGX
     &                       +lg(YZ,i,j,kc)*BSRICCI_DXGY 
     &                       +lg(ZZ,i,j,kc)*BSRICCI_DXGZ)
#else
      BSRICCI_TERM2 =  0.5D0*(ADM_BS_gxx(i,j,k)*BSRICCI_DZGX
     &                       +ADM_BS_gxy(i,j,k)*BSRICCI_DZGY 
     &                       +ADM_BS_gxz(i,j,k)*BSRICCI_DZGZ 
     &                       +ADM_BS_gxz(i,j,k)*BSRICCI_DXGX
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DXGY 
     &                       +ADM_BS_gzz(i,j,k)*BSRICCI_DXGZ)
#endif
      BSRICCI_TERM3 =  0.5D0*(BSRICCI_GX*BSCHR1_XZX
     &                       +BSRICCI_GY*BSCHR1_XZY
     &                       +BSRICCI_GZ*BSCHR1_XZZ
     &                       +BSRICCI_GX*BSCHR1_ZXX
     &                       +BSRICCI_GY*BSCHR1_ZXY
     &                       +BSRICCI_GZ*BSCHR1_ZXZ) 
      BSRICCI_RXZ = BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3 +
     &(2*BSCHR2_XXZ*BSCHR1_XXX + 
     & BSCHR2_YXZ*(BSCHR1_XXY + BSCHR1_YXX) + BSCHR2_XXX*BSCHR1_ZXX + 
     & BSCHR2_ZXZ*(BSCHR1_XXZ + BSCHR1_ZXX) + BSCHR2_YXX*BSCHR1_ZXY + 
     & BSCHR2_ZXX*BSCHR1_ZXZ)*BSUPPERMET_UXX + 
     &(2*(BSCHR2_XYZ*BSCHR1_XXX + BSCHR2_XXZ*BSCHR1_XXY) + 
     & BSCHR2_YYZ*(BSCHR1_XXY + BSCHR1_YXX) + 
     & BSCHR2_YXZ*(BSCHR1_XYY + BSCHR1_YXY) + BSCHR2_XXY*BSCHR1_ZXX + 
     & BSCHR2_ZYZ*(BSCHR1_XXZ + BSCHR1_ZXX) + (BSCHR2_XXX + BSCHR2_YXY)*BSCHR1_ZXY + 
     & BSCHR2_ZXZ*(BSCHR1_XYZ + BSCHR1_ZXY) + BSCHR2_ZXY*BSCHR1_ZXZ + 
     & BSCHR2_YXX*BSCHR1_ZYY + BSCHR2_ZXX*BSCHR1_ZYZ)*BSUPPERMET_UXY + 
     & (BSCHR2_ZXZ*BSCHR1_XZZ + BSCHR2_YZZ*(BSCHR1_XXY + BSCHR1_YXX) + 
     & BSCHR2_XXZ*BSCHR1_ZXX + BSCHR2_ZZZ*(BSCHR1_XXZ + BSCHR1_ZXX) + 
     & BSCHR2_YXZ*(BSCHR1_XYZ + BSCHR1_YXZ + BSCHR1_ZXY) + BSCHR2_XXX*BSCHR1_ZXZ + 
     & 2*(BSCHR2_XZZ*BSCHR1_XXX + BSCHR2_XXZ*BSCHR1_XXZ + BSCHR2_ZXZ*BSCHR1_ZXZ) + 
     & BSCHR2_YXX*BSCHR1_ZYZ + BSCHR2_ZXX*BSCHR1_ZZZ)*BSUPPERMET_UXZ + 
     & (2*BSCHR2_XYZ*BSCHR1_XXY + 
     & BSCHR2_YYZ*(BSCHR1_XYY + BSCHR1_YXY) + BSCHR2_XXY*BSCHR1_ZXY + 
     & BSCHR2_ZYZ*(BSCHR1_XYZ + BSCHR1_ZXY) + BSCHR2_YXY*BSCHR1_ZYY + 
     & BSCHR2_ZXY*BSCHR1_ZYZ)*BSUPPERMET_UYY + 
     & (2*(BSCHR2_XZZ*BSCHR1_XXY + BSCHR2_XYZ*BSCHR1_XXZ) + 
     & (BSCHR2_YYZ + BSCHR2_ZZZ)*BSCHR1_XYZ + BSCHR2_YZZ*(BSCHR1_XYY + BSCHR1_YXY) + 
     & BSCHR2_YYZ*BSCHR1_YXZ + (BSCHR2_XXZ + BSCHR2_ZZZ)*BSCHR1_ZXY + 
     & BSCHR2_XXY*BSCHR1_ZXZ + BSCHR2_ZYZ*(BSCHR1_XZZ + BSCHR1_ZXZ) + 
     & BSCHR2_YXZ*BSCHR1_ZYY + (BSCHR2_YXY + BSCHR2_ZXZ)*BSCHR1_ZYZ + 
     & BSCHR2_ZXY*BSCHR1_ZZZ)*BSUPPERMET_UYZ + 
     & (2*BSCHR2_XZZ*BSCHR1_XXZ + 
     & BSCHR2_YZZ*(BSCHR1_XYZ + BSCHR1_YXZ) + BSCHR2_XXZ*BSCHR1_ZXZ + 
     & BSCHR2_ZZZ*(BSCHR1_XZZ + BSCHR1_ZXZ) + BSCHR2_YXZ*BSCHR1_ZYZ + 
     & BSCHR2_ZXZ*BSCHR1_ZZZ)*BSUPPERMET_UZZ

      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGYY
#ifdef OPT
      BSRICCI_TERM2 =  0.5D0*(lg(XY,i,j,kc)*BSRICCI_DYGX
     &                       +lg(YY,i,j,kc)*BSRICCI_DYGY 
     &                       +lg(YZ,i,j,kc)*BSRICCI_DYGZ 
     &                       +lg(XY,i,j,kc)*BSRICCI_DYGX
     &                       +lg(YY,i,j,kc)*BSRICCI_DYGY 
     &                       +lg(YZ,i,j,kc)*BSRICCI_DYGZ)
#else
      BSRICCI_TERM2 =  0.5D0*(ADM_BS_gxy(i,j,k)*BSRICCI_DYGX
     &                       +ADM_BS_gyy(i,j,k)*BSRICCI_DYGY 
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DYGZ 
     &                       +ADM_BS_gxy(i,j,k)*BSRICCI_DYGX
     &                       +ADM_BS_gyy(i,j,k)*BSRICCI_DYGY 
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DYGZ)
#endif
      BSRICCI_TERM3 =  0.5D0*(BSRICCI_GX*BSCHR1_YYX
     &                       +BSRICCI_GY*BSCHR1_YYY
     &                       +BSRICCI_GZ*BSCHR1_YYZ
     &                       +BSRICCI_GX*BSCHR1_YYX
     &                       +BSRICCI_GY*BSCHR1_YYY
     &                       +BSRICCI_GZ*BSCHR1_YYZ) 
      BSRICCI_RYY = BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3 +
     & (BSCHR2_XXY*(BSCHR1_XXY + 2*BSCHR1_YXX) + 
     & 3*BSCHR2_YXY*BSCHR1_YXY + BSCHR2_ZXY*
     & (2*BSCHR1_YXZ + BSCHR1_ZXY))*BSUPPERMET_UXX + 
     & (BSCHR2_XYY*(BSCHR1_XXY + 2*BSCHR1_YXX) + 
     & BSCHR2_XXY*(BSCHR1_XYY + 2*BSCHR1_YXY) + 
     & 3*(BSCHR2_YYY*BSCHR1_YXY + BSCHR2_YXY*BSCHR1_YYY) + 
     & 2*(BSCHR2_ZYY*BSCHR1_YXZ + BSCHR2_ZXY*BSCHR1_YYZ) 
     & + BSCHR2_ZYY*BSCHR1_ZXY + 
     & BSCHR2_ZXY*BSCHR1_ZYY)*BSUPPERMET_UXY + 
     &(BSCHR2_XYZ*(BSCHR1_XXY + 2*BSCHR1_YXX) + 
     & BSCHR2_XXY*(BSCHR1_XYZ + 2*BSCHR1_YXZ) + 
     & 3*(BSCHR2_YYZ*BSCHR1_YXY + BSCHR2_YXY*BSCHR1_YYZ) + 
     & 2*(BSCHR2_ZYZ*BSCHR1_YXZ + BSCHR2_ZXY*BSCHR1_YZZ) 
     & + BSCHR2_ZYZ*BSCHR1_ZXY + 
     & BSCHR2_ZXY*BSCHR1_ZYZ)*BSUPPERMET_UXZ + 
     & (BSCHR2_XYY*(BSCHR1_XYY + 2*BSCHR1_YXY) + 
     & 3*BSCHR2_YYY*BSCHR1_YYY + BSCHR2_ZYY*
     & (2*BSCHR1_YYZ + BSCHR1_ZYY))*BSUPPERMET_UYY + 
     &(BSCHR2_XYZ*(BSCHR1_XYY + 2*BSCHR1_YXY) + 
     & BSCHR2_XYY*(BSCHR1_XYZ + 2*BSCHR1_YXZ) + 
     & 3*(BSCHR2_YYZ*BSCHR1_YYY + BSCHR2_YYY*BSCHR1_YYZ) + 
     & 2*(BSCHR2_ZYZ*BSCHR1_YYZ + BSCHR2_ZYY*BSCHR1_YZZ) + BSCHR2_ZYZ*BSCHR1_ZYY + 
     & BSCHR2_ZYY*BSCHR1_ZYZ)*BSUPPERMET_UYZ + 
     &(BSCHR2_XYZ*(BSCHR1_XYZ + 2*BSCHR1_YXZ) + 
     & 3*BSCHR2_YYZ*BSCHR1_YYZ + BSCHR2_ZYZ*(2*BSCHR1_YZZ + BSCHR1_ZYZ))*BSUPPERMET_UZZ


      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGYZ
#ifdef OPT
      BSRICCI_TERM2 =  0.5D0*(lg(XY,i,j,kc)*BSRICCI_DZGX
     &                       +lg(YY,i,j,kc)*BSRICCI_DZGY 
     &                       +lg(YZ,i,j,kc)*BSRICCI_DZGZ 
     &                       +lg(XZ,i,j,kc)*BSRICCI_DYGX
     &                       +lg(YZ,i,j,kc)*BSRICCI_DYGY 
     &                       +lg(ZZ,i,j,kc)*BSRICCI_DYGZ)
#else
      BSRICCI_TERM2 =  0.5D0*(ADM_BS_gxy(i,j,k)*BSRICCI_DZGX
     &                       +ADM_BS_gyy(i,j,k)*BSRICCI_DZGY 
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DZGZ 
     &                       +ADM_BS_gxz(i,j,k)*BSRICCI_DYGX
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DYGY 
     &                       +ADM_BS_gzz(i,j,k)*BSRICCI_DYGZ)
#endif
      BSRICCI_TERM3 =  0.5D0*(BSRICCI_GX*BSCHR1_YZX
     &                       +BSRICCI_GY*BSCHR1_YZY
     &                       +BSRICCI_GZ*BSCHR1_YZZ
     &                       +BSRICCI_GX*BSCHR1_ZYX
     &                       +BSRICCI_GY*BSCHR1_ZYY
     &                       +BSRICCI_GZ*BSCHR1_ZYZ) 
      BSRICCI_RYZ = BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3 +
     & (BSCHR2_XXZ*(BSCHR1_XXY + BSCHR1_YXX) + 
     & 2*BSCHR2_YXZ*BSCHR1_YXY + BSCHR2_XXY*BSCHR1_ZXX + BSCHR2_YXY*BSCHR1_ZXY + 
     & BSCHR2_ZXZ*(BSCHR1_YXZ + BSCHR1_ZXY) + BSCHR2_ZXY*BSCHR1_ZXZ)*BSUPPERMET_UXX + 
     & (BSCHR2_XYZ*(BSCHR1_XXY + BSCHR1_YXX) + 
     & BSCHR2_XXZ*(BSCHR1_XYY + BSCHR1_YXY) + 
     & 2*(BSCHR2_YYZ*BSCHR1_YXY + BSCHR2_YXZ*BSCHR1_YYY) + BSCHR2_XYY*BSCHR1_ZXX + 
     & (BSCHR2_XXY + BSCHR2_YYY)*BSCHR1_ZXY + BSCHR2_ZYZ*(BSCHR1_YXZ + BSCHR1_ZXY) + 
     & BSCHR2_ZYY*BSCHR1_ZXZ + BSCHR2_YXY*BSCHR1_ZYY + 
     & BSCHR2_ZXZ*(BSCHR1_YYZ + BSCHR1_ZYY) + BSCHR2_ZXY*BSCHR1_ZYZ)*BSUPPERMET_UXY + 
     & (BSCHR2_XZZ*(BSCHR1_XXY + BSCHR1_YXX) + 
     & BSCHR2_XXZ*(BSCHR1_XYZ + BSCHR1_YXZ) + 
     & 2*(BSCHR2_YZZ*BSCHR1_YXY + BSCHR2_YXZ*BSCHR1_YYZ) + BSCHR2_XYZ*BSCHR1_ZXX + 
     & BSCHR2_YYZ*BSCHR1_ZXY + BSCHR2_ZZZ*(BSCHR1_YXZ + BSCHR1_ZXY) + 
     & (BSCHR2_XXY + BSCHR2_ZYZ)*BSCHR1_ZXZ + BSCHR2_YXY*BSCHR1_ZYZ + 
     & BSCHR2_ZXZ*(BSCHR1_YZZ + BSCHR1_ZYZ) + BSCHR2_ZXY*BSCHR1_ZZZ)*BSUPPERMET_UXZ + 
     & (BSCHR2_XYZ*(BSCHR1_XYY + BSCHR1_YXY) + 
     & 2*BSCHR2_YYZ*BSCHR1_YYY + BSCHR2_XYY*BSCHR1_ZXY + BSCHR2_YYY*BSCHR1_ZYY + 
     & BSCHR2_ZYZ*(BSCHR1_YYZ + BSCHR1_ZYY) + BSCHR2_ZYY*BSCHR1_ZYZ)*BSUPPERMET_UYY + 
     & (BSCHR2_XZZ*(BSCHR1_XYY + BSCHR1_YXY) + BSCHR2_ZYZ*BSCHR1_YZZ + 
     & BSCHR2_XYZ*(BSCHR1_XYZ + BSCHR1_YXZ + BSCHR1_ZXY) + BSCHR2_XYY*BSCHR1_ZXZ + 
     & BSCHR2_YYZ*BSCHR1_ZYY + BSCHR2_ZZZ*(BSCHR1_YYZ + BSCHR1_ZYY) + 
     & BSCHR2_YYY*BSCHR1_ZYZ + 2*(BSCHR2_YZZ*BSCHR1_YYY + BSCHR2_YYZ*BSCHR1_YYZ + 
     & BSCHR2_ZYZ*BSCHR1_ZYZ) + BSCHR2_ZYY*BSCHR1_ZZZ)*BSUPPERMET_UYZ + 
     & (BSCHR2_XZZ*(BSCHR1_XYZ + BSCHR1_YXZ) + 
     & 2*BSCHR2_YZZ*BSCHR1_YYZ + BSCHR2_XYZ*BSCHR1_ZXZ + BSCHR2_YYZ*BSCHR1_ZYZ + 
     & BSCHR2_ZZZ*(BSCHR1_YZZ + BSCHR1_ZYZ) + BSCHR2_ZYZ*BSCHR1_ZZZ)*BSUPPERMET_UZZ


      BSRICCI_TERM1 = -0.5D0*WAVEBSG_DDGZZ
#ifdef OPT
      BSRICCI_TERM2 =  0.5D0*(lg(XZ,i,j,kc)*BSRICCI_DZGX
     &                       +lg(YZ,i,j,kc)*BSRICCI_DZGY 
     &                       +lg(ZZ,i,j,kc)*BSRICCI_DZGZ 
     &                       +lg(XZ,i,j,kc)*BSRICCI_DZGX
     &                       +lg(YZ,i,j,kc)*BSRICCI_DZGY 
     &                       +lg(ZZ,i,j,kc)*BSRICCI_DZGZ)
#else
      BSRICCI_TERM2 =  0.5D0*(ADM_BS_gxz(i,j,k)*BSRICCI_DZGX
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DZGY 
     &                       +ADM_BS_gzz(i,j,k)*BSRICCI_DZGZ 
     &                       +ADM_BS_gxz(i,j,k)*BSRICCI_DZGX
     &                       +ADM_BS_gyz(i,j,k)*BSRICCI_DZGY 
     &                       +ADM_BS_gzz(i,j,k)*BSRICCI_DZGZ)
#endif
      BSRICCI_TERM3 =  0.5D0*(BSRICCI_GX*BSCHR1_ZZX
     &                       +BSRICCI_GY*BSCHR1_ZZY
     &                       +BSRICCI_GZ*BSCHR1_ZZZ
     &                       +BSRICCI_GX*BSCHR1_ZZX
     &                       +BSRICCI_GY*BSCHR1_ZZY
     &                       +BSRICCI_GZ*BSCHR1_ZZZ) 
      BSRICCI_RZZ =  BSRICCI_TERM1+BSRICCI_TERM2+BSRICCI_TERM3
     & +(BSCHR2_XXZ*(BSCHR1_XXZ + 2*BSCHR1_ZXX) + 
     & BSCHR2_YXZ*(BSCHR1_YXZ + 2*BSCHR1_ZXY) + 3*BSCHR2_ZXZ*BSCHR1_ZXZ)*BSUPPERMET_UXX + 
     & (BSCHR2_XYZ*(BSCHR1_XXZ + 2*BSCHR1_ZXX) + 
     & BSCHR2_XXZ*(BSCHR1_XYZ + 2*BSCHR1_ZXY) + 
     & BSCHR2_YYZ*(BSCHR1_YXZ + 2*BSCHR1_ZXY) + 
     & BSCHR2_YXZ*(BSCHR1_YYZ + 2*BSCHR1_ZYY) + 
     & 3*(BSCHR2_ZYZ*BSCHR1_ZXZ + BSCHR2_ZXZ*BSCHR1_ZYZ))*BSUPPERMET_UXY + 
     &(BSCHR2_XZZ*(BSCHR1_XXZ + 2*BSCHR1_ZXX) + 
     & BSCHR2_YZZ*(BSCHR1_YXZ + 2*BSCHR1_ZXY) + 
     & BSCHR2_XXZ*(BSCHR1_XZZ + 2*BSCHR1_ZXZ) + 
     & BSCHR2_YXZ*(BSCHR1_YZZ + 2*BSCHR1_ZYZ) + 
     & 3*(BSCHR2_ZZZ*BSCHR1_ZXZ + BSCHR2_ZXZ*BSCHR1_ZZZ))*BSUPPERMET_UXZ + 
     & (BSCHR2_XYZ*(BSCHR1_XYZ + 2*BSCHR1_ZXY) + 
     & BSCHR2_YYZ*(BSCHR1_YYZ + 2*BSCHR1_ZYY) + 3*BSCHR2_ZYZ*BSCHR1_ZYZ)*BSUPPERMET_UYY + 
     & (BSCHR2_XZZ*(BSCHR1_XYZ + 2*BSCHR1_ZXY) + 
     & BSCHR2_XYZ*(BSCHR1_XZZ + 2*BSCHR1_ZXZ) + 
     & BSCHR2_YZZ*(BSCHR1_YYZ + 2*BSCHR1_ZYY) + 
     & BSCHR2_YYZ*(BSCHR1_YZZ + 2*BSCHR1_ZYZ) + 
     & 3*(BSCHR2_ZZZ*BSCHR1_ZYZ + BSCHR2_ZYZ*BSCHR1_ZZZ))*BSUPPERMET_UYZ + 
     &(BSCHR2_XZZ*(BSCHR1_XZZ + 2*BSCHR1_ZXZ) + 
     & BSCHR2_YZZ*(BSCHR1_YZZ + 2*BSCHR1_ZYZ) + 3*BSCHR2_ZZZ*BSCHR1_ZZZ)*BSUPPERMET_UZZ

      else

      BSRICCI_RXX = (BSDDG_DXYDGXY+(-BSDDG_DXXDGYY-BSDDG_DYYDGXX)*0.5 -
     & BSCHR1_XYY*BSCHR2_XXX +BSCHR1_XXY*BSCHR2_XXY-BSCHR1_YYY*BSCHR2_YXX
     & + BSCHR1_YXY*BSCHR2_YXY - BSCHR1_ZYY*BSCHR2_ZXX +
     & BSCHR1_ZXY*BSCHR2_ZXY)*BSUPPERMET_UYY + (-BSDDG_DXXDGYZ
     & + BSDDG_DXYDGXZ +
     & BSDDG_DXZDGXY - BSDDG_DYZDGXX - 2*(BSCHR1_YYZ*BSCHR2_YXX +
     & BSCHR1_ZYZ*BSCHR2_ZXX) + 2*(-(BSCHR1_XYZ*BSCHR2_XXX)
     & + BSCHR1_XXZ*BSCHR2_XXY +
     & BSCHR1_YXZ*BSCHR2_YXY + BSCHR1_ZXZ*BSCHR2_ZXY))*BSUPPERMET_UYZ
     & + (BSDDG_DXZDGXZ
     & + (-BSDDG_DXXDGZZ - BSDDG_DZZDGXX)*0.5 - BSCHR1_XZZ*BSCHR2_XXX +
     & BSCHR1_XXZ*BSCHR2_XXZ - BSCHR1_YZZ*BSCHR2_YXX
     & + BSCHR1_YXZ*BSCHR2_YXZ -
     & BSCHR1_ZZZ*BSCHR2_ZXX + BSCHR1_ZXZ*BSCHR2_ZXZ)*BSUPPERMET_UZZ


      BSRICCI_RXY = (-BSDDG_DXYDGXY + (BSDDG_DXXDGYY + BSDDG_DYYDGXX)*0.5
     & + BSCHR1_XYY*BSCHR2_XXX - BSCHR1_XXY*BSCHR2_XXY
     & + BSCHR1_YYY*BSCHR2_YXX -
     & BSCHR1_YXY*BSCHR2_YXY + BSCHR1_ZYY*BSCHR2_ZXX -
     & BSCHR1_ZXY*BSCHR2_ZXY)*BSUPPERMET_UXY + ((BSDDG_DXXDGYZ
     & - BSDDG_DXYDGXZ -
     & BSDDG_DXZDGXY + BSDDG_DYZDGXX)*0.5 + BSCHR1_XYZ*BSCHR2_XXX -
     & BSCHR1_XXZ*BSCHR2_XXY + BSCHR1_YYZ*BSCHR2_YXX
     & - BSCHR1_YXZ*BSCHR2_YXY +
     & BSCHR1_ZYZ*BSCHR2_ZXX - BSCHR1_ZXZ*BSCHR2_ZXY)*BSUPPERMET_UXZ +
     & ((-BSDDG_DXYDGYZ + BSDDG_DXZDGYY + BSDDG_DYYDGXZ
     & - BSDDG_DYZDGXY)*0.5 -
     & BSCHR1_XYZ*BSCHR2_XXY + BSCHR1_XYY*BSCHR2_XXZ
     & - BSCHR1_YYZ*BSCHR2_YXY +
     & BSCHR1_YYY*BSCHR2_YXZ - BSCHR1_ZYZ*BSCHR2_ZXY +
     & BSCHR1_ZYY*BSCHR2_ZXZ)*BSUPPERMET_UYZ + ((-BSDDG_DXYDGZZ
     & + BSDDG_DXZDGYZ +
     & BSDDG_DYZDGXZ - BSDDG_DZZDGXY)*0.5 - BSCHR1_XZZ*BSCHR2_XXY +
     & BSCHR1_XYZ*BSCHR2_XXZ - BSCHR1_YZZ*BSCHR2_YXY
     & + BSCHR1_YYZ*BSCHR2_YXZ -
     & BSCHR1_ZZZ*BSCHR2_ZXY + BSCHR1_ZYZ*BSCHR2_ZXZ)*BSUPPERMET_UZZ 


      BSRICCI_RXZ = ((BSDDG_DXXDGYZ - BSDDG_DXYDGXZ - BSDDG_DXZDGXY +
     & BSDDG_DYZDGXX)*0.5 + BSCHR1_XYZ*BSCHR2_XXX - BSCHR1_XXZ*BSCHR2_XXY +
     & BSCHR1_YYZ*BSCHR2_YXX - BSCHR1_YXZ*BSCHR2_YXY
     & + BSCHR1_ZYZ*BSCHR2_ZXX -
     & BSCHR1_ZXZ*BSCHR2_ZXY)*BSUPPERMET_UXY + (-BSDDG_DXZDGXZ
     & + (BSDDG_DXXDGZZ +
     & BSDDG_DZZDGXX)*0.5 + BSCHR1_XZZ*BSCHR2_XXX - BSCHR1_XXZ*BSCHR2_XXZ +
     & BSCHR1_YZZ*BSCHR2_YXX - BSCHR1_YXZ*BSCHR2_YXZ
     & + BSCHR1_ZZZ*BSCHR2_ZXX -
     & BSCHR1_ZXZ*BSCHR2_ZXZ)*BSUPPERMET_UXZ + ((BSDDG_DXYDGYZ
     & - BSDDG_DXZDGYY -
     & BSDDG_DYYDGXZ + BSDDG_DYZDGXY)*0.5 + BSCHR1_XYZ*BSCHR2_XXY -
     & BSCHR1_XYY*BSCHR2_XXZ + BSCHR1_YYZ*BSCHR2_YXY
     & - BSCHR1_YYY*BSCHR2_YXZ +
     & BSCHR1_ZYZ*BSCHR2_ZXY - BSCHR1_ZYY*BSCHR2_ZXZ)*BSUPPERMET_UYY
     & + ((BSDDG_DXYDGZZ
     & - BSDDG_DXZDGYZ - BSDDG_DYZDGXZ + BSDDG_DZZDGXY)*0.5
     & + BSCHR1_XZZ*BSCHR2_XXY
     & - BSCHR1_XYZ*BSCHR2_XXZ + BSCHR1_YZZ*BSCHR2_YXY
     & - BSCHR1_YYZ*BSCHR2_YXZ +
     & BSCHR1_ZZZ*BSCHR2_ZXY - BSCHR1_ZYZ*BSCHR2_ZXZ)*BSUPPERMET_UYZ 


      BSRICCI_RYY = (BSDDG_DXYDGXY + (-BSDDG_DXXDGYY - BSDDG_DYYDGXX)*0.5 -
     & BSCHR1_XYY*BSCHR2_XXX + BSCHR1_XXY*BSCHR2_XXY
     & - BSCHR1_YYY*BSCHR2_YXX +
     & BSCHR1_YXY*BSCHR2_YXY - BSCHR1_ZYY*BSCHR2_ZXX +
     & BSCHR1_ZXY*BSCHR2_ZXY)*BSUPPERMET_UXX + (BSDDG_DXYDGYZ
     & - BSDDG_DXZDGYY -
     & BSDDG_DYYDGXZ + BSDDG_DYZDGXY + 2*(BSCHR1_XYZ*BSCHR2_XXY -
     & BSCHR1_XYY*BSCHR2_XXZ + BSCHR1_YYZ*BSCHR2_YXY
     & + BSCHR1_ZYZ*BSCHR2_ZXY) -
     & 2*(BSCHR1_YYY*BSCHR2_YXZ + BSCHR1_ZYY*BSCHR2_ZXZ)) *BSUPPERMET_UXZ +
     & (BSDDG_DYZDGYZ + (-BSDDG_DYYDGZZ - BSDDG_DZZDGYY)*0.5 -
     & BSCHR1_XZZ*BSCHR2_XYY + BSCHR1_XYZ*BSCHR2_XYZ
     & - BSCHR1_YZZ*BSCHR2_YYY +
     & BSCHR1_YYZ*BSCHR2_YYZ - BSCHR1_ZZZ*BSCHR2_ZYY +
     & BSCHR1_ZYZ*BSCHR2_ZYZ)*BSUPPERMET_UZZ 


      BSRICCI_RYZ = ((-BSDDG_DXXDGYZ + BSDDG_DXYDGXZ + BSDDG_DXZDGXY -
     & BSDDG_DYZDGXX)*0.5 - BSCHR1_XYZ*BSCHR2_XXX + BSCHR1_XXZ*BSCHR2_XXY -
     & BSCHR1_YYZ*BSCHR2_YXX + BSCHR1_YXZ*BSCHR2_YXY
     & - BSCHR1_ZYZ*BSCHR2_ZXX +
     & BSCHR1_ZXZ*BSCHR2_ZXY)*BSUPPERMET_UXX + ((-BSDDG_DXYDGYZ
     & + BSDDG_DXZDGYY +
     & BSDDG_DYYDGXZ - BSDDG_DYZDGXY)*0.5 - BSCHR1_XYZ*BSCHR2_XXY +
     & BSCHR1_XYY*BSCHR2_XXZ - BSCHR1_YYZ*BSCHR2_YXY
     & + BSCHR1_YYY*BSCHR2_YXZ -
     & BSCHR1_ZYZ*BSCHR2_ZXY + BSCHR1_ZYY*BSCHR2_ZXZ)*BSUPPERMET_UXY
     & + ((BSDDG_DXYDGZZ
     & - BSDDG_DXZDGYZ - BSDDG_DYZDGXZ + BSDDG_DZZDGXY)*0.5
     & + BSCHR1_XZZ*BSCHR2_XXY
     & - BSCHR1_XYZ*BSCHR2_XXZ + BSCHR1_YZZ*BSCHR2_YXY
     & - BSCHR1_YYZ*BSCHR2_YXZ +
     & BSCHR1_ZZZ*BSCHR2_ZXY - BSCHR1_ZYZ*BSCHR2_ZXZ)*BSUPPERMET_UXZ
     & + (-BSDDG_DYZDGYZ
     & + (BSDDG_DYYDGZZ + BSDDG_DZZDGYY)*0.5 + BSCHR1_XZZ*BSCHR2_XYY -
     & BSCHR1_XYZ*BSCHR2_XYZ + BSCHR1_YZZ*BSCHR2_YYY
     & - BSCHR1_YYZ*BSCHR2_YYZ +
     & BSCHR1_ZZZ*BSCHR2_ZYY - BSCHR1_ZYZ*BSCHR2_ZYZ)*BSUPPERMET_UYZ 


      BSRICCI_RZZ = (BSDDG_DXZDGXZ + (-BSDDG_DXXDGZZ - BSDDG_DZZDGXX)*0.5 -
     & BSCHR1_XZZ*BSCHR2_XXX + BSCHR1_XXZ*BSCHR2_XXZ
     & - BSCHR1_YZZ*BSCHR2_YXX +
     & BSCHR1_YXZ*BSCHR2_YXZ - BSCHR1_ZZZ*BSCHR2_ZXX +
     & BSCHR1_ZXZ*BSCHR2_ZXZ)*BSUPPERMET_UXX + (-BSDDG_DXYDGZZ
     & + BSDDG_DXZDGYZ +
     & BSDDG_DYZDGXZ - BSDDG_DZZDGXY - 2*(BSCHR1_YZZ*BSCHR2_YXY +
     & BSCHR1_ZZZ*BSCHR2_ZXY) + 2*(-(BSCHR1_XZZ*BSCHR2_XXY)
     & + BSCHR1_XYZ*BSCHR2_XXZ +
     & BSCHR1_YYZ*BSCHR2_YXZ + BSCHR1_ZYZ*BSCHR2_ZXZ))*BSUPPERMET_UXY
     & + (BSDDG_DYZDGYZ
     & + (-BSDDG_DYYDGZZ - BSDDG_DZZDGYY)*0.5 - BSCHR1_XZZ*BSCHR2_XYY +
     & BSCHR1_XYZ*BSCHR2_XYZ - BSCHR1_YZZ*BSCHR2_YYY
     & + BSCHR1_YYZ*BSCHR2_YYZ -
     & BSCHR1_ZZZ*BSCHR2_ZYY + BSCHR1_ZYZ*BSCHR2_ZYZ)*BSUPPERMET_UYY 

      end if

#endif
